@@ -25,6 +25,9 @@
 
 #include <GeoIP.h>
 #include <GeoIPCity.h>
+#ifdef PHP_WIN32
+#include <GeoIP_internal.h>
+#endif
 
 #include "php.h"
 #include "php_ini.h"
@@ -56,6 +59,14 @@ zend_function_entry geoip_functions[] = {
 #if LIBGEOIP_VERSION >= 1004001
 	PHP_FE(geoip_region_name_by_code,	NULL)
 	PHP_FE(geoip_time_zone_by_country_and_region,	NULL)
+#endif
+#ifdef HAVE_CUSTOM_DIRECTORY
+    PHP_FE(geoip_setup_custom_directory,	NULL)
+#endif
+	PHP_FE(geoip_asnum_by_name,   NULL)
+	PHP_FE(geoip_domain_by_name,   NULL)
+#if LIBGEOIP_VERSION >= 1004008
+	PHP_FE(geoip_netspeedcell_by_name, NULL)
 #endif
 	{NULL, NULL, NULL}
 };
@@ -85,11 +96,53 @@ zend_module_entry geoip_module_entry = {
 ZEND_GET_MODULE(geoip)
 #endif
 
+#ifdef HAVE_CUSTOM_DIRECTORY
+/* {{{ geoip_change_custom_directory() helper function
+ */
+static void geoip_change_custom_directory(char *value)
+{
+#if LIBGEOIP_VERSION >= 1004007
+	GeoIP_cleanup();
+#else
+	int i;
+	if (GeoIPDBFileName != NULL) {
+		for (i = 0; i < NUM_DB_TYPES; i++) {
+			if (GeoIPDBFileName[i]) {
+				free(GeoIPDBFileName[i]);
+			}
+		}
+		free(GeoIPDBFileName);
+		GeoIPDBFileName = NULL;
+	}
+#endif
+
+	GeoIP_setup_custom_directory(value);
+	_GeoIP_setup_dbfilename();
+}
+/* }}} */
+#endif
+
+#ifdef HAVE_CUSTOM_DIRECTORY
+/* {{{ PHP_INI_MH 
++ */
+static PHP_INI_MH(OnUpdateDirectory)
+{
+	if (stage == PHP_INI_STAGE_RUNTIME || stage == PHP_INI_STAGE_HTACCESS) {
+		GEOIP_G(set_runtime_custom_directory) = 1;
+		geoip_change_custom_directory(new_value);
+		return SUCCESS;
+	}
+	
+	return OnUpdateString(entry, new_value, new_value_length, mh_arg1, mh_arg2, mh_arg3, stage TSRMLS_CC);
+}
+/* }}} */
+#endif
+
 /* {{{ PHP_INI
  */
 PHP_INI_BEGIN()
 #ifdef HAVE_CUSTOM_DIRECTORY
-	STD_PHP_INI_ENTRY("geoip.custom_directory", NULL, PHP_INI_ALL, OnUpdateString, custom_directory, zend_geoip_globals, geoip_globals)
+	STD_PHP_INI_ENTRY("geoip.custom_directory", NULL, PHP_INI_ALL, OnUpdateDirectory, custom_directory, zend_geoip_globals, geoip_globals)
 #endif
 PHP_INI_END()
 /* }}} */
@@ -99,6 +152,7 @@ PHP_INI_END()
 static void php_geoip_init_globals(zend_geoip_globals *geoip_globals)
 {
 	geoip_globals->custom_directory = NULL;
+	geoip_globals->set_runtime_custom_directory = 0;
 }
 /* }}} */
 
@@ -114,7 +168,6 @@ PHP_MINIT_FUNCTION(geoip)
 #ifdef HAVE_CUSTOM_DIRECTORY
 	GeoIP_setup_custom_directory(GEOIP_G(custom_directory));
 #endif
-	
 	_GeoIP_setup_dbfilename();
 	
 	/* For database type constants */
@@ -129,7 +182,10 @@ PHP_MINIT_FUNCTION(geoip)
 	REGISTER_LONG_CONSTANT("GEOIP_ASNUM_EDITION",       GEOIP_ASNUM_EDITION,       CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("GEOIP_NETSPEED_EDITION",    GEOIP_NETSPEED_EDITION,    CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("GEOIP_DOMAIN_EDITION",      GEOIP_DOMAIN_EDITION,      CONST_CS | CONST_PERSISTENT);
-	
+#if LIBGEOIP_VERSION >= 1004008
+	REGISTER_LONG_CONSTANT("GEOIP_NETSPEED_EDITION_REV1",GEOIP_NETSPEED_EDITION_REV1,CONST_CS | CONST_PERSISTENT);
+#endif
+
 	/* For netspeed constants */
 	REGISTER_LONG_CONSTANT("GEOIP_UNKNOWN_SPEED",       GEOIP_UNKNOWN_SPEED,       CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("GEOIP_DIALUP_SPEED",        GEOIP_DIALUP_SPEED,        CONST_CS | CONST_PERSISTENT);
@@ -162,6 +218,15 @@ PHP_RINIT_FUNCTION(geoip)
  */
 PHP_RSHUTDOWN_FUNCTION(geoip)
 {
+#ifdef HAVE_CUSTOM_DIRECTORY
+	/* If we have a custom directory (and have support from   */
+	/* libgeoip, we reset the extension to default directory) */
+	if (GEOIP_G(set_runtime_custom_directory)) {
+		geoip_change_custom_directory(GEOIP_G(custom_directory));
+		GEOIP_G(set_runtime_custom_directory) = 0;
+	}
+#endif
+	
 	return SUCCESS;
 }
 /* }}} */
@@ -237,7 +302,9 @@ PHP_FUNCTION(geoip_db_get_all_info)
 			array_init(row);
 
 			add_assoc_bool(row, "available", GeoIP_db_avail(i));
-			add_assoc_string(row, "description", (char *)GeoIPDBDescription[i], 1);
+			if (GeoIPDBDescription[i]) {
+				add_assoc_string(row, "description", (char *)GeoIPDBDescription[i], 1);
+			}
 			if (GeoIPDBFileName[i]) {
 				add_assoc_string(row, "filename", GeoIPDBFileName[i], 1);
 			}
@@ -307,7 +374,6 @@ PHP_FUNCTION(geoip_database_info)
 		return_code = c_func(gi, hostname); \
 		GeoIP_delete(gi); \
 		if (return_code == NULL) { \
-			php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname); \
 			RETURN_FALSE; \
 		} \
 		RETURN_STRING((char*)return_code, 1); \
@@ -340,7 +406,6 @@ PHP_FUNCTION(geoip_continent_code_by_name)
 	id = GeoIP_id_by_name(gi, hostname);
 	GeoIP_delete(gi);
 	if (id == 0) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
 		RETURN_FALSE;
 	}
 	RETURN_STRING((char *)GeoIP_country_continent[id], 1);
@@ -370,7 +435,6 @@ PHP_FUNCTION(geoip_org_by_name)
 	org = GeoIP_org_by_name(gi, hostname);
 	GeoIP_delete(gi);
 	if (org == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
 		RETURN_FALSE;
 	}
 	RETVAL_STRING(org, 1);
@@ -378,6 +442,98 @@ PHP_FUNCTION(geoip_org_by_name)
 }
 /* }}} */
 
+/* {{{ proto string geoip_asnum_by_name( string hostname )
+   Returns the Domain Name found in the GeoIP Database */
+PHP_FUNCTION(geoip_asnum_by_name)
+{
+	GeoIP * gi;
+	char * hostname = NULL;
+	char * org;
+	int arglen;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
+		return;
+	}
+
+	if (GeoIP_db_avail(GEOIP_ASNUM_EDITION)) {
+		gi = GeoIP_open_type(GEOIP_ASNUM_EDITION, GEOIP_STANDARD);
+	}   else {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Required database not available at %s.", GeoIPDBFileName[GEOIP_ASNUM_EDITION]);
+		return;
+	}
+
+	org = GeoIP_name_by_name(gi, hostname);
+	GeoIP_delete(gi);
+	if (org == NULL) {
+		RETURN_FALSE;
+	}
+	RETVAL_STRING(org, 1);
+	free(org);
+}
+/* }}} */
+
+/* {{{ proto string geoip_domain_by_name( string hostname )
+   Returns the Domain Name found in the GeoIP Database */
+PHP_FUNCTION(geoip_domain_by_name)
+{
+	GeoIP * gi;
+	char * hostname = NULL;
+	char * org;
+	int arglen;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
+		return;
+	}
+
+	if (GeoIP_db_avail(GEOIP_DOMAIN_EDITION)) {
+		gi = GeoIP_open_type(GEOIP_DOMAIN_EDITION, GEOIP_STANDARD);
+	}   else {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Required database not available at %s.", GeoIPDBFileName[GEOIP_DOMAIN_EDITION]);
+		return;
+	}
+
+	org = GeoIP_name_by_name(gi, hostname);
+	GeoIP_delete(gi);
+	if (org == NULL) {
+		RETURN_FALSE;
+	}
+	RETVAL_STRING(org, 1);
+	free(org);
+}
+/* }}} */
+
+#if LIBGEOIP_VERSION >= 1004008
+/* {{{ proto string geoip_netspeedcell_by_name( string hostname )
+   Returns the Net Speed found in the GeoIP Database */
+PHP_FUNCTION(geoip_netspeedcell_by_name)
+{
+	GeoIP * gi;
+	char * hostname = NULL;
+	char * org;
+	int arglen;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
+		return;
+	}
+
+	if (GeoIP_db_avail(GEOIP_NETSPEED_EDITION_REV1)) {
+		gi = GeoIP_open_type(GEOIP_NETSPEED_EDITION_REV1, GEOIP_STANDARD);
+	}   else {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Required database not available at %s.", GeoIPDBFileName[GEOIP_NETSPEED_EDITION_REV1]);
+		return;
+	}
+
+	org = GeoIP_name_by_name(gi, hostname);
+	GeoIP_delete(gi);
+	if (org == NULL) {
+		RETURN_FALSE;
+	}
+	RETVAL_STRING(org, 1);
+	free(org);
+}
+/* }}} */
+#endif
+
 /* {{{ proto array geoip_record_by_name( string hostname )
    Returns the detailed City information found in the GeoIP Database */
 PHP_FUNCTION(geoip_record_by_name)
@@ -406,7 +562,6 @@ PHP_FUNCTION(geoip_record_by_name)
 	GeoIP_delete(gi);
 	
 	if (NULL == gir) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
 		RETURN_FALSE;
 	}
 	
@@ -487,7 +642,6 @@ PHP_FUNCTION(geoip_region_by_name)
 	GeoIP_delete(gi);
 
 	if (NULL == region) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
 		RETURN_FALSE;
 	}
 
@@ -522,7 +676,6 @@ PHP_FUNCTION(geoip_isp_by_name)
 	isp = GeoIP_name_by_name(gi, hostname);
 	GeoIP_delete(gi);
 	if (isp == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
 		RETURN_FALSE;
 	}
 	RETVAL_STRING(isp, 1);
@@ -585,6 +738,25 @@ PHP_FUNCTION(geoip_time_zone_by_country_and_region)
 /* }}} */
 #endif
 
+#ifdef HAVE_CUSTOM_DIRECTORY
+/* {{{ proto void geoip_setup_custom_directory( string directory )
+   Sets the custom directory for GeoIP databases */
+PHP_FUNCTION(geoip_setup_custom_directory)
+{
+	char * dir = NULL;
+	int dirlen;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &dir, &dirlen) == FAILURE) {
+		return;
+	}
+
+	GEOIP_G(set_runtime_custom_directory) = 1;
+	geoip_change_custom_directory(dir);
+}
+/* }}} */
+#endif
+
+
 /*
  * Local variables:
  * tab-width: 4
